# -*- coding: utf-8 -*-

from functools import partial

from django import forms
from django.contrib import admin, messages
from django.contrib.admin.helpers import ActionForm
from django.utils.translation import ugettext_lazy as _


class ThuxActionForm(ActionForm):
    action = forms.ChoiceField(label=_('Action:'))
    select_across = forms.BooleanField(
        label='',
        required=False,
        initial=0,
        widget=forms.HiddenInput({'class': 'select-across'}),
    )

    def do(self, request, queryset):
        raise Exception(_("method 'do' not implemented"))

    def form_valid(self, request, queryset):
        return self.do(request, queryset)

    def form_invalid(self, request):
        for error in self.errors:
            for detail in self.errors.get(error):
                messages.error(request, '{}: {}'.format(error, detail))


class ActionClassMixin:
    action_list_dict = []

    class Media:
        js = ("admin/js/jquery.init.js", "admin/js/action_form.js",)

    def __init__(self, *args, **kwargs):
        self.set_action_list_dict()
        self.set_thux_actions()
        super(ActionClassMixin, self).__init__(*args, **kwargs)

    def get_action_names(self, request):
        """
        Return valid action name list
        """
        return [action[0] for action in self.get_action_choices(request) if action[0]]

    def get_action_classes(self):
        action_classes = getattr(self, 'action_classes', None)
        return action_classes

    def set_action_list_dict(self):
        self.action_list_dict = []
        for action_class in self.get_action_classes():
            self.action_list_dict.append({
                'action_class': action_class,
                'action_form': action_class.form,
                'action_name': action_class.action_name,
                'action_description': action_class.short_description
            })
            self.actions.append(action_class.action_name)

    def get_action_forms(self, request):
        """
        Return valid action form list
        """
        action_form_list = []
        for action_dict in self.action_list_dict:
            ActionForm = action_dict['action_form']
            ActionForm.action_name = action_dict['action_name']
            action_form = ActionForm(auto_id=None)
            action_form.fields['action'].choices = self.get_action_choices(request)
            action_form_list.append(action_form)
        return action_form_list

    def changelist_view(self, request, extra_context=None):
        response = super().changelist_view(request, extra_context=extra_context)
        try:
            action_form_list = self.get_action_forms(request)
            response.context_data.update({'action_forms':  action_form_list})
        except Exception as e:
            pass
        return response

    def set_action(self, request, queryset, action_dict):
        action_form = action_dict['action_form']
        action_name = action_dict['action_name']
        form = action_form()
        action_choices = self.get_action_choices(request)  # ModelAdmin method
        form.fields['action'].choices = action_choices
        if 'action' in request.POST and action_name in request.POST['action']:
            form = action_form(request.POST)
            form.fields['action'].choices = action_choices
            if form.is_valid():
                return form.form_valid(request, queryset)
            else:
                return form.form_invalid(request)

    def set_thux_actions(self):
        for action_dict in self.action_list_dict:
            action_name = action_dict['action_name']
            description = action_dict['action_description']
            partial_action = partial(self.__class__.set_action, action_dict=action_dict)
            partial_action.short_description = description
            setattr(self.__class__, action_name, partial_action)
